; $Header: /home/campbell/Languages/Scheme/scm/x-scm/RCS/ol.scm,v 1.3 1992/08/09 03:42:29 campbell Beta $
;
; OpenLook definitions (from <Xol/OpenLook.h>)

(define ol:absent-pair		0)
(define ol:all			1)
(define ol:always		2)
(define ol:atom-help		3)
(define ol:both			4)
(define ol:bottom		5)
(define ol:buttonstack		6)
(define ol:center		7)
(define ol:class-help		8)
(define ol:columns		9)
(define ol:copy-mask-value	10)
(define ol:copy-size		11)
(define ol:copy-source-value	12)
(define ol:current		13)
(define ol:default-pair		14)
(define ol:disk-source		15)
(define ol:display-form		16)
(define ol:down			17)
(define ol:existing-source	18)
(define ol:fixedcols		19)
(define ol:fixedheight		20)
(define ol:fixedrows		21)
(define ol:fixedwidth		22)
(define ol:flat-button		23)
(define ol:flat-checkbox	24)
(define ol:flat-container	25)
(define ol:flat-exclusives	26)
(define ol:flat-help		27)
(define ol:flat-nonexclusives	28)
(define ol:halfstack		29)
(define ol:horizontal		30)
(define ol:image		31)
(define ol:in			32)
(define ol:indirect-source	33)
(define ol:label		34)
(define ol:left			35)
(define ol:mask-pair		36)
(define ol:maximize		37)
(define ol:millimeters		38)
(define ol:minimize		39)
(define ol:never		40)
(define ol:next			41)
(define ol:none			42)
(define ol:nonebottom		43)
(define ol:noneleft		44)
(define ol:noneright		45)
(define ol:nonetop		46)
(define ol:notices		47)
(define ol:no-virtual-mapping	48)
(define ol:oblong		49)
(define ol:out			50)
(define ol:override-pair	51)
(define ol:pixels		52)
(define ol:points		53)
(define ol:popup		54)
(define ol:previous		55)
(define ol:prog-defined-source	56)
(define ol:rectbutton		57)
(define ol:right		58)
(define ol:rows			59)
(define ol:source-form		60)
(define ol:source-pair		61)
(define ol:stayup		62)
(define ol:string		63)
(define ol:string-source	64)
(define ol:text-append		65)
(define ol:text-edit		66)
(define ol:text-read		67)
(define ol:top			68)
(define ol:transparent-source	69)
(define ol:vertical		70)
(define ol:virtual-button	71)
(define ol:virtual-key		72)
(define ol:widget-help		73)
(define ol:window-help		74)
(define ol:wrap-any		75)
(define ol:wrap-white-space	76)
(define ol:continuous		77)
(define ol:granularity		78)
(define ol:release		79)
(define ol:tickmark		80)
(define ol:percent		81)
(define ol:slidervalue		82)
(define ol:wt-base		83)
(define ol:wt-cmd		84)
(define ol:wt-notice		85)
(define ol:wt-help		86)
(define ol:wt-other		87)
(define ol:success		88)
(define ol:duplicate-key	89)
(define ol:duplicatekey		89)
(define ol:bad-key		90)
(define ol:menu-full		91)
(define ol:menu-limited		92)
(define ol:menu-cancel		93)
(define ol:selectkey		94)
(define ol:menukey		95)
(define ol:menudefault		96)
(define ol:menudefaultkey	97)
(define ol:hsbmenu		98)
(define ol:vsbmenu		99)
(define ol:adjustkey		100)
(define ol:nextapp		101)
(define ol:nextwindow		102)
(define ol:prevapp		103)
(define ol:prevwindow		104)
(define ol:windowmenu		105)
(define ol:workspacemenu	106)
(define ol:defaultaction	108)
(define ol:drag			109)
(define ol:drop			110)
(define ol:togglepushpin	111)
(define ol:pageleft		112)
(define ol:pageright		113)
(define ol:scrollbottom		114)
(define ol:scrolltop		115)
(define ol:multiright		116)
(define ol:multileft		117)
(define ol:multidown		118)
(define ol:multiup		119)
(define ol:immediate		120)
(define ol:moveup		121)
(define ol:movedown		122)
(define ol:moveright		123)
(define ol:moveleft		124)
(define ol:click-to-type	125)
(define ol:realestate		126)
(define ol:underline		127)
(define ol:highlight		128)
(define ol:inactive		129)
(define ol:display		130)
(define ol:proc			131)
(define ol:size-proc		132)
(define ol:draw-proc		133)
(define ol:pinned-menu		134)
(define ol:press-drag-menu	135)
(define ol:stayup-menu		136)
(define ol:pointer		137)
(define ol:inputfocus		138)
(define ol:quit			142)
(define ol:destroy		143)
(define ol:dismiss		144)
(define ol:pre			145)
(define ol:post			146)

; Other things that are mysteriously missing from Xol/*.h on my system

(define xt:n-align-captions		"alignCaptions")
(define xt:n-control-area		"controlArea")
(define xt:n-emanate-widget		"emanateWidget")
(define xt:n-gravity			"gravity")
(define xt:n-label-justify		"labelJustify")
(define xt:n-layout-type		"layoutType")
(define xt:n-max-label			"maxLabel")
(define xt:n-measure			"measure")
(define xt:n-menu-pane			"menuPane")
(define xt:n-min-label			"minLabel")
(define xt:n-same-size			"sameSize")
(define xt:n-select			"select")
(define xt:n-slider-value		"sliderValue")
(define xt:n-text-area			"textArea")
(define xt:n-weight			"weight")
(define xt:n-x-offset			"xOffset")
(define xt:n-x-add-width		"xAddWidth")
(define xt:n-x-vary-offset		"xVaryOffset")
(define xt:n-x-resizable		"xResizable")
(define xt:n-x-attach-right		"xAttachRight")
(define xt:n-x-attach-offset		"xAttachOffset")
(define xt:n-y-ref-name			"yRefName")
(define xt:n-y-ref-widget		"yRefWidget")
(define xt:n-y-offset			"yOffset")
(define xt:n-y-add-height		"yAddHeight")
(define xt:n-y-vary-offset		"yVaryOffset")
(define xt:n-y-resizable		"yResizable")
(define xt:n-y-attach-bottom		"yAttachBottom")
(define xt:n-y-attach-offset		"yAttachOffset")
