; $Header: /home/campbell/Languages/Scheme/scm/x-scm/RCS/xt.scm,v 1.2 1992/07/04 05:00:36 campbell Beta $
;
; Xt definitions.  This file may ultimately contain all Xt-related
; definitions, but at present contains only resource name symbols
; taken  from StringDefs.h
;
;  Author: Larry Campbell (campbell@redsox.bsw.com)
; 
;  Copyright 1992 by The Boston Software Works, Inc.
;  Permission to use for any purpose whatsoever granted, as long
;  as this copyright notice remains intact.  Please send bug fixes
;  or enhancements to the above email address.
;
(define xt:n-accelerators "accelerators")
(define xt:n-allow-horiz "allowHoriz")
(define xt:n-allow-shell-resize "allowShellResize")
(define xt:n-allow-vert "allowVert")
(define xt:n-ancestor-sensitive "ancestorSensitive")
(define xt:n-background "background")
(define xt:n-background-pixmap "backgroundPixmap")
(define xt:n-bitmap "bitmap")
(define xt:n-border-color "borderColor")
(define xt:n-border "borderColor")
(define xt:n-border-pixmap "borderPixmap")
(define xt:n-border-width "borderWidth")
(define xt:n-callback "callback")
(define xt:n-children "children")
(define xt:n-colormap "colormap")
(define xt:n-depth "depth")
(define xt:n-destroy-callback "destroyCallback")
(define xt:n-edit-type "editType")
(define xt:n-file "file")
(define xt:n-font "font")
(define xt:n-force-bars "forceBars")
(define xt:n-foreground "foreground")
(define xt:n-function "function")
(define xt:n-height "height")
(define xt:n-highlight "highlight")
(define xt:n-h-space "hSpace")
(define xt:n-index "index")
(define xt:n-initial-resources-persistent "initialResourcesPersistent")
(define xt:n-inner-height "innerHeight")
(define xt:n-inner-width "innerWidth")
(define xt:n-inner-window "innerWindow")
(define xt:n-insert-position "insertPosition")
(define xt:n-internal-height "internalHeight")
(define xt:n-internal-width "internalWidth")
(define xt:n-jump-proc "jumpProc")
(define xt:n-justify "justify")
(define xt:n-knob-height "knobHeight")
(define xt:n-knob-indent "knobIndent")
(define xt:n-knob-pixel "knobPixel")
(define xt:n-knob-width "knobWidth")
(define xt:n-label "label")
(define xt:n-length "length")
(define xt:n-lower-right "lowerRight")
(define xt:n-mapped-when-managed "mappedWhenManaged")
(define xt:n-menu-entry "menuEntry")
(define xt:n-name "name")
(define xt:n-notify "notify")
(define xt:n-num-children "numChildren")
(define xt:n-orientation "orientation")
(define xt:n-parameter "parameter")
(define xt:n-pixmap "pixmap")
(define xt:n-popup-callback "popupCallback")
(define xt:n-popdown-callback "popdownCallback")
(define xt:n-resize "resize")
(define xt:n-reverse-video "reverseVideo")
(define xt:n-screen "screen")
(define xt:n-scroll-proc "scrollProc")
(define xt:n-scroll-d-cursor "scrollDCursor")
(define xt:n-scroll-h-cursor "scrollHCursor")
(define xt:n-scroll-l-cursor "scrollLCursor")
(define xt:n-scroll-r-cursor "scrollRCursor")
(define xt:n-scroll-u-cursor "scrollUCursor")
(define xt:n-scroll-v-cursor "scrollVCursor")
(define xt:n-selection "selection")
(define xt:n-selection-array "selectionArray")
(define xt:n-sensitive "sensitive")
(define xt:n-shown "shown")
(define xt:n-space "space")
(define xt:n-string "string")
(define xt:n-text-options "textOptions")
(define xt:n-text-sink "textSink")
(define xt:n-text-source "textSource")
(define xt:n-thickness "thickness")
(define xt:n-thumb "thumb")
(define xt:n-thumb-proc "thumbProc")
(define xt:n-title "title")
(define xt:n-top "top")
(define xt:n-translations "translations")
(define xt:n-unrealize-callback "unrealizeCallback")
(define xt:n-update "update")
(define xt:n-use-bottom "useBottom")
(define xt:n-use-right "useRight")
(define xt:n-value "value")
(define xt:n-v-space "vSpace")
(define xt:n-width "width")
(define xt:n-window "window")
(define xt:n-x "x")
(define xt:n-y "y")
