; $Header: /home/campbell/Languages/Scheme/scm/x-scm/RCS/xm.scm,v 1.2 1992/07/08 03:20:26 campbell Beta $
;
; Various definitions from Xm.h.
;
;  Author: Larry Campbell (campbell@redsox.bsw.com)
; 
;  Copyright 1992 by The Boston Software Works, Inc.
;  Permission to use for any purpose whatsoever granted, as long
;  as this copyright notice remains intact.  Please send bug fixes
;  or enhancements to the above email address.
;
(define xm:n-accelerators "accelerators")
(define xm:n-accelerator-text "acceleratorText")
(define xm:n-adjustLast "adjustLast")
(define xm:n-adjust-margin "adjustMargin")
(define xm:n-alignment "alignment")
(define xm:n-allow-horiz "allowHoriz")
(define xm:n-allow-overlap "allowOverlap")
(define xm:n-allow-shell-resize "allowShellResize")
(define xm:n-allow-vert "allowVert")
(define xm:n-arm-callback "armCallback")
(define xm:n-arrow-direction "arrowDirection")
(define xm:n-ancestor-sensitive "ancestorSensitive")
(define xm:n-background "background")
(define xm:n-background-pixmap "backgroundPixmap")
(define xm:n-bitmap "bitmap")
(define xm:n-border-color "borderColor")
(define xm:n-border "borderColor")
(define xm:n-border-pixmap "borderPixmap")
(define xm:n-border-width "borderWidth")
(define xm:n-browse-selection-callback "browseSelectionCallback")
(define xm:n-callback "callback")
(define xm:n-children "children")
(define xm:n-colormap "colormap")
(define xm:n-columns "columns")
(define xm:n-decimal-points "decimalPoints")
(define xm:n-delete-response "deleteResponse")
(define xm:n-depth "depth")
(define xm:n-destroy-callback "destroyCallback")
(define xm:n-disarm-callback "disarmCallback")
(define xm:n-drag-callback "dragCallback")
(define xm:n-edit-mode "editMode")
(define xm:n-edit-type "editType")
(define xm:n-editable "editable")
(define xm:n-expose-callback "exposeCallback")
(define xm:n-file "file")
(define xm:n-font "font")
(define xm:n-force-bars "forceBars")
(define xm:n-foreground "foreground")
(define xm:n-function "function")
(define xm:n-height "height")
(define xm:n-highlight "highlight")
(define xm:n-highlight-thickness "highlightThickness")
(define xm:n-h-space "hSpace")
(define xm:n-index "index")
(define xm:n-initial-resources-persistent "initialResourcesPersistent")
(define xm:n-inner-height "innerHeight")
(define xm:n-inner-width "innerWidth")
(define xm:n-inner-window "innerWindow")
(define xm:n-insert-position "insertPosition")
(define xm:n-internal-height "internalHeight")
(define xm:n-internal-width "internalWidth")
(define xm:n-items "items")
(define xm:n-item-count "itemCount")
(define xm:n-jump-proc "jumpProc")
(define xm:n-justify "justify")
(define xm:n-knob-height "knobHeight")
(define xm:n-knob-indent "knobIndent")
(define xm:n-knob-pixel "knobPixel")
(define xm:n-knob-width "knobWidth")
(define xm:n-label "label")
(define xm:n-label-string "labelString")
(define xm:n-length "length")
(define xm:n-lower-right "lowerRight")
(define xm:n-mapped-when-managed "mappedWhenManaged")
(define xm:n-mapping-delay "mappingDelay")
(define xm:n-margin-height "marginHeight")
(define xm:n-margin-top "marginTop")
(define xm:n-margin-bottom "marginBottom")
(define xm:n-margin-width "marginWidth")
(define xm:n-margin-right "marginRight")
(define xm:n-margin-left "marginLeft")
(define xm:n-maximum "maximum")
(define xm:n-menu-accelerator "menuAccelerator")
(define xm:n-menu-bar "menuBar")
(define xm:n-menu-cursor "menuCursor")
(define xm:n-menu-entry "menuEntry")
(define xm:n-menu-help-widget "menuHelpWidget")
(define xm:n-minimum "minimum")
(define xm:n-multiple-selection-callback "multipleSelectionCallback")
(define xm:n-name "name")
(define xm:n-notify "notify")
(define xm:n-num-children "numChildren")
(define xm:n-num-columns "numColumns")
(define xm:n-orientation "orientation")
(define xm:n-packing "packing")
(define xm:n-parameter "parameter")
(define xm:n-pixmap "pixmap")
(define xm:n-popup-callback "popupCallback")
(define xm:n-popdown-callback "popdownCallback")
(define xm:n-processing-direction "processingDirection")
(define xm:n-recompute-size "recomputeSize")
(define xm:n-resize "resize")
(define xm:n-resize-callback "resizeCallback")
(define xm:n-reverse-video "reverseVideo")
(define xm:n-row-column-type "rowColumnType")
(define xm:n-rows "rows")
(define xm:n-scale-height "scaleHeight")
(define xm:n-scale-multiple "scaleMultiple")
(define xm:n-scale-width "scaleWidth")
(define xm:n-screen "screen")
(define xm:n-scroll-proc "scrollProc")
(define xm:n-scroll-d-cursor "scrollDCursor")
(define xm:n-scroll-h-cursor "scrollHCursor")
(define xm:n-scroll-l-cursor "scrollLCursor")
(define xm:n-scroll-r-cursor "scrollRCursor")
(define xm:n-scroll-u-cursor "scrollUCursor")
(define xm:n-scroll-v-cursor "scrollVCursor")
(define xm:n-selected-items "selectedItems")
(define xm:n-selected-item-count "selectedItemCount")
(define xm:n-selection "selection")
(define xm:n-selection-array "selectionArray")
(define xm:n-selection-policy "selectionPolicy")
(define xm:n-sensitive "sensitive")
(define xm:n-set "set")
(define xm:n-shadow-thickness "shadowThickness")
(define xm:n-show-as-default "showAsDefault")
(define xm:n-show-separator "showSeparator")
(define xm:n-show-value "showValue")
(define xm:n-shown "shown")
(define xm:n-single-selection-callback "singleSelectionCallback")
(define xm:n-space "space")
(define xm:n-spacing "spacing")
(define xm:n-string "string")
(define xm:n-string-direction "stringDirection")
(define xm:n-sub-menu-id "subMenuId")
(define xm:n-text-options "textOptions")
(define xm:n-text-sink "textSink")
(define xm:n-text-source "textSource")
(define xm:n-text-string "textString")
(define xm:n-thickness "thickness")
(define xm:n-thumb "thumb")
(define xm:n-thumb-proc "thumbProc")
(define xm:n-title "title")
(define xm:n-title-string "titleString")
(define xm:n-top "top")
(define xm:n-translations "translations")
(define xm:n-traversal-on "traversalOn")
(define xm:n-unrealize-callback "unrealizeCallback")
(define xm:n-update "update")
(define xm:n-use-bottom "useBottom")
(define xm:n-use-right "useRight")
(define xm:n-value "value")
(define xm:n-value-changed-callback "valueChangedCallback")
(define xm:n-visible-item-count "visibleItemCount")
(define xm:n-visible-when-off "visibleWhenOff")
(define xm:n-v-space "vSpace")
(define xm:n-which-button "whichButton")
(define xm:n-width "width")
(define xm:n-window "window")
(define xm:n-x "x")
(define xm:n-y "y")

; Other miscellaneous definitions from Xm.h

(define xm:n-activate-callback "activateCallback")

(define xm:work-area	       0)
(define xm:menu-bar	       1)
(define xm:menu-pulldown       2)
(define xm:menu-popup          3)
(define xm:menu-option         4)


; Menu defines

(define xm:no-orientation	0)
(define xm:vertical		1)
(define xm:horizontal		2)

(define xm:work-area		0)
(define xm:menu-bar		1)
(define xm:menu-pulldown	2)
(define xm:menu-popup		3)
(define xm:menu-option		4)

(define xm:no-packing		0)
(define xm:pack-tight		1)
(define xm:pack-column		2)
(define xm:pack-none		3)


; Label defines

(define xm:alignment-beginning	0)
(define xm:alignment-center	1)
(define xm:alignment-end	2)


; ToggleButton defines

(define xm:n-of-many		1)
(define xm:one-of-many		2)

; Form constraint resources

(define xm:n-top-attachment	"topAttachment")
(define xm:n-bottom-attachment	"bottomAttachment")
(define xm:n-left-attachment	"leftAttachment")
(define xm:n-right-attachment	"rightAttachment")

(define xm:n-top-widget		"topWidget")
(define xm:n-bottom-widget	"bottomWidget")
(define xm:n-left-widget	"leftWidget")
(define xm:n-right-widget	"rightWidget")

(define xm:n-top-position	"topPosition")
(define xm:n-bottom-position	"bottomPosition")
(define xm:n-left-position	"leftPosition")
(define xm:n-right-position	"rightPosition")
(define xm:n-top-offset		"topOffset")
(define xm:n-bottom-offset	"bottomOffset")
(define xm:n-left-offset	"leftOffset")
(define xm:n-right-offset	"rightOffset")

(define xm:n-resizable		"resizable")


; Form defines

(define xm:attach-none			0)
(define xm:attach-form			1)
(define xm:attach-opposite-form		2)
(define xm:attach-widget		3)
(define xm:attach-opposite-widget	4)
(define xm:attach-position		5)
(define xm:attach-self			6)

(define xm:resize-none	0)
(define xm:resize-grow	1)
(define xm:resize-any	2)

; XmMessageBox           stuff not common to other dialogs

(define xm:n-default-button-type	"defaultButtonType")
(define xm:n-minimize-buttons		"minimizeButtons")
(define xm:n-message-string		"messageString")
(define xm:n-message-alignment		"messageAlignment")
(define xm:n-symbol-pixmap		"symbolPixmap")
(define xm:n-dialog-type		"dialogType")

; DeleteResponse values
(define xm:destroy 	0)
(define xm:unmap 	1)
(define xm:do-nothing	2)

; defines for dialog type

(define xm:dialog-error        1)
(define xm:dialog-information  2)
(define xm:dialog-message      3)
(define xm:dialog-question     4)
(define xm:dialog-warning      5)
(define xm:dialog-working      6)


; defines for callbacks

(define xm:n-apply-callback		"applyCallback")
(define xm:n-cancel-callback		"cancelCallback")
(define xm:n-command-entered-callback	"commandEnteredCallback")
(define xm:n-command-changed-callback	"commandChangedCallback")
(define xm:n-default-action-callback	"defaultActionCallback")
(define xm:n-ok-callback		"okCallback")
(define xm:n-no-match-callback		"noMatchCallback")


; XmN defines for subwidgets

(define xm:n-ok-label-string		"okLabelString")
(define xm:n-cancel-label-string	"cancelLabelString")
(define xm:n-help-label-string		"helpLabelString")
(define xm:n-apply-label-string		"applyLabelString")
(define xm:n-selection-label-string	"selectionLabelString")
(define xm:n-list-label-string		"listLabelString")
(define xm:n-prompt-string		"promptString")

; Text mode stuff

(define xm:multi-line-edit	0)
(define xm:single-line-edit	1)

(define xm:n-auto-show-cursor-position	"autoShowCursorPosition")
(define xm:n-resize-width		"resizeWidth")
(define xm:n-scroll-horizontal		"scrollHorizontal")
(define xm:n-scroll-vertical		"scrollVertical")

; ArrowButton stuff

(define xm:arrow-up	0)
(define xm:arrow-down	1)
(define xm:arrow-left	2)
(define xm:arrow-right	3)

; Selection types
 
(define xm:single-select	0)
(define xm:multiple-select	1)
(define xm:extended-select	2)
(define xm:browse-select	3)

; Scrolled Window defines.

(define xm:n-horizontal-scroll-bar	"horizontalScrollBar")
(define xm:n-vertical-scroll-bar	"verticalScrollBar")
(define xm:n-work-window		"workWindow")
(define xm:n-clip-window		"clipWindow")
(define xm:n-scrolling-policy		"scrollingPolicy")
(define xm:n-visual-policy		"visualPolicy")
(define xm:n-scroll-bar-display-policy	"scrollBarDisplayPolicy")
(define xm:n-scroll-bar-placement	"scrollBarPlacement")
(define xm:n-update-slider-size		"updateSliderSize")
(define xm:n-scrolled-window-margin-height	"scrolledWindowMarginHeight")
(define xm:n-scrolled-window-margin-width	"scrolledWindowMarginWidth")

(define xm:variable	        0)
(define xm:constant	        1)
(define xm:resize-if-possible	2)

(define xm:automatic		0)
(define xm:application-defined	1)

(define xm:as-needed		1)

; Defined values for scrollbar

(define xm:max-on-top		0)
(define xm:max-on-bottom	1)
(define xm:max-on-left		2)
(define xm:max-on-right		3)


; Resources for vertical paned window

(define xm:n-refigure-mode		"refigureMode")
(define xm:n-separator-on		"separatorOn")
(define xm:n-sash-indent		"sashIndent")
(define xm:n-sash-width			"sashWidth")
(define xm:n-sash-height		"sashHeight")
(define xm:n-sash-shadow-thickness	"sashShadowThickness")


; Constraint Resources for Vertical Paned Window

(define xm:n-allow-resize	"allowResize")
(define xm:n-skip-adjust	"skipAdjust")
(define xm:n-pane-minimum       "paneMinimum")
(define xm:n-pane-maximum       "paneMaximum")

