; $Header: /home/campbell/Languages/Scheme/scm/x-scm/RCS/xw.scm,v 1.1 1992/07/03 03:06:52 campbell Beta $
;
; Definitions from Xw.h

(define xt:n-traversal-on		"traversalOn")
(define xt:n-traversal-type		"traversalType")
(define xt:n-highlight-style		"highlightStyle")
(define xt:n-highlight-tile		"highlightTile")
(define xt:n-highlight-thickness	"highlightThickness")
(define xt:n-highlight-color		"highlightColor")
(define xt:n-background-tile		"backgroundTile")
(define xt:n-cursor			"cursor")
(define xt:n-recompute-size		"recomputeSize")
(define xt:n-layout			"layout")
(define xt:n-label-location		"labelLocation")
(define xt:n-sensitive-tile		"sensitiveTile")
(define xt:n-columns			"columns")
(define xt:n-mode			"mode")
(define xt:n-set			"set")
(define xt:n-select			"select")
(define xt:n-release			"release")
(define xt:n-next-top			"nextTop")
(define xt:n-title-showing		"titleShowing")
(define xt:n-mgr-title-override		"mgrTitleOverride")
(define xt:n-title-type			"titleType")
(define xt:n-title-string		"titleString")
(define xt:n-title-image		"titleImage")
(define xt:n-font-color			"fontColor")
(define xt:n-mnemonic			"mnemonic")
(define xt:n-underline-title		"underlineTitle")
(define xt:n-mgr-underline-override	"mgrUnderlineOverride")
(define xt:n-underline-position		"underlinePosition")
(define xt:n-attach-to			"attachTo")
(define xt:n-kbd-accelerator		"kbdAccelerator")
(define xt:n-associate-children		"associateChildren")
(define xt:n-menu-post			"menuPost")
(define xt:n-menu-select		"menuSelect")
(define xt:n-post-accelerator		"postAccelerator")
(define xt:n-menu-unpost		"menuUnpost")
(define xt:n-kbd-select			"kbdSelect")
(define xt:n-num-columns		"numColumns")
(define xt:n-row-position		"rowPosition")
(define xt:n-column-position		"columnPosition")
(define xt:n-selection-method		"selectionMethod")
(define xt:n-element-highlight		"elementHighlight")
(define xt:n-selection-bias		"selectionBias")
(define xt:n-selection-style		"selectionStyle")
(define xt:n-column-width		"columnWidth")
(define xt:n-element-height		"elementHeight")
(define xt:n-selected-elements		"selectedElements")
(define xt:n-numSelected-elements	"numSelectedElements")
(define xt:n-destroy-mode		"destroyMode")
(define xt:n-layout-type		"layoutType")
(define xt:n-force-size			"forceSize")
(define xt:n-single-row			"singleRow")
(define xt:n-separator-type		"separatorType")
(define xt:n-vsb-x			"vsbX")
(define xt:n-vsb-y			"vsbY")
(define xt:n-vsb-width			"vsbWidth")
(define xt:n-vsb-height			"vsbHeight")
(define xt:n-hsb-x			"hsbX")
(define xt:n-hsb-y			"hsbY")
(define xt:n-hsb-width			"hsbWidth")
(define xt:n-hsb-height			"hsbHeight")
(define xt:n-v-slider-min		"vSliderMin")
(define xt:n-v-slider-max		"vSliderMax")
(define xt:n-v-slider-origin		"vSliderOrigin")
(define xt:n-v-slider-extent		"vSliderExtent")
(define xt:n-h-slider-min		"hSliderMin")
(define xt:n-h-slider-max		"hSliderMax")
(define xt:n-h-slider-origin		"hSliderOrigin")
(define xt:n-h-slider-extent		"hSliderExtent")
(define xt:n-h-scroll-event		"hScrollEvent")
(define xt:n-v-scroll-event		"vScrollEvent")
(define xt:n-v-scroll-bar-width		"vScrollBarWidth")
(define xt:n-v-scroll-bar-height	"vScrollBarHeight")
(define xt:n-h-scroll-bar-width		"hScrollBarWidth")
(define xt:n-h-scroll-bar-height	"hScrollBarHeight")
(define xt:n-force-vertical-sb		"forceVerticalSB")
(define xt:n-force-horizontal-sb	"forceHorizontalSB")
(define xt:n-initial-x			"initialX")
(define xt:n-initial-y			"initialY")
(define xt:n-border-pad			"borderPad")
(define xt:n-s-r-image			"rasterImage")
(define xt:n-show-selected		"showSelected")
(define xt:n-display-position		"displayPosition")
(define xt:n-insert-position		"insertPosition")
(define xt:n-left-margin		"leftMargin")
(define xt:n-right-margin		"rightMargin")
(define xt:n-top-margin			"topMargin")
(define xt:n-bottom-margin		"bottomMargin")
(define xt:n-selection-array		"selectionArray")
(define xt:n-text-source		"textSource")
(define xt:n-selection			"selection")
(define xt:n-maximum-size		"maximumSize")
(define xt:n-edit-type			"editType")
(define xt:n-file			"file")
(define xt:n-string			"string")
(define xt:n-length			"length")
(define xt:n-font			"font")
(define xt:n-disk-src			"disksrc")
(define xt:n-string-src			"stringsrc")
(define xt:n-execute			"execute")
(define xt:n-source-type		"sourceType")
(define xt:n-motion-verification	"motionVerification")
(define xt:n-modify-verification	"modifyVerification")
(define xt:n-leave-verification		"leaveVerification")
(define xt:n-wrap			"wrap")
(define xt:n-wrap-form			"wrapForm")
(define xt:n-wrap-break			"wrapBreak")
(define xt:n-scroll			"scroll")
(define xt:n-grow			"grow")
(define xt:n-alignment			"alignment")
(define xt:n-line-space			"lineSpace")
(define xt:n-gravity			"gravity")
(define xt:n-slider-min			"sliderMin")
(define xt:n-slider-max			"sliderMax")
(define xt:n-slider-origin		"sliderOrigin")
(define xt:n-slider-extent		"sliderExtent")
(define xt:n-slider-color		"sliderColor")
(define xt:n-slide-orientation		"slideOrientation")
(define xt:n-slide-move-type		"slideMoveType")
(define xt:n-slide-area-tile		"slideAreaTile")
(define xt:n-slider-moved		"sliderMoved")
(define xt:n-area-selected		"areaSelected")
(define xt:n-slider-tile		"sliderTile")
(define xt:n-sliderReleased		"sliderReleased")
(define xt:n-xRef-name			"xRefName")
(define xt:n-xRef-widget		"xRefWidget")
(define xt:n-x-offset			"xOffset")
(define xt:n-x-add-width		"xAddWidth")
(define xt:n-x-vary-offset		"xVaryOffset")
(define xt:n-x-resizable		"xResizable")
(define xt:n-x-attach-right		"xAttachRight")
(define xt:n-x-attach-offset		"xAttachOffset")
(define xt:n-y-ref-name			"yRefName")
(define xt:n-y-ref-widget		"yRefWidget")
(define xt:n-y-offset			"yOffset")
(define xt:n-y-add-height		"yAddHeight")
(define xt:n-y-vary-offset		"yVaryOffset")
(define xt:n-y-resizable		"yResizable")
(define xt:n-y-attach-bottom		"yAttachBottom")
(define xt:n-y-attach-offset		"yAttachOffset")
(define xt:n-pixel-scale		"pixelScale")
(define xt:n-grid-thickness		"gridThickness")
(define xt:n-image			"image")
(define xt:n-draw-color			"drawColor")
(define xt:n-erase-color		"eraseColor")
(define xt:n-erase-on			"eraseOn")
(define xt:n-label-type			"labelType")
(define xt:n-label-image		"labelImage")
(define xt:n-cascade-image		"cascadeImage")
(define xt:n-mark-image			"markImage")
(define xt:n-set-mark			"setMark")
(define xt:n-cascade-on			"cascadeOn")
(define xt:n-invert-on-enter		"invertOnEnter")
(define xt:n-mgr-override-mnemonic	"mgrOverrideMnemonic")
(define xt:n-cascade-select		"cascadeSelect")
(define xt:n-cascadeUnselect		"cascadeUnselect")
(define xt:n-menu-mgr-id		"menuMgrId")
(define xt:n-scrollbar-orientation	"scrollbarOrientation")
(define xt:n-selection-color		"selectionColor")
(define xt:n-initial-delay		"initialDelay")
(define xt:n-repeat-rate		"repeatRate")
(define xt:n-granularity		"granularity")
(define xt:n-invert-on-select		"invertOnSelect")
(define xt:n-toggle			"toggle")
(define xt:n-square			"square")
(define xt:n-select-color		"selectColor")
(define xt:n-allow-resize		"allowResize")
(define xt:n-sash-indent		"sashIndent")
(define xt:n-refigure-mode		"refigureMode")
(define xt:n-padding			"padding")
(define xt:n-min			"min")
(define xt:n-max			"max")
(define xt:n-skip-adjust		"skipAdjust")
(define xt:n-framed			"framed")
(define xt:n-border-frame		"borderFrame")
(define xt:n-expose			"expose")
(define xt:n-resize			"resize")
(define xt:n-key-down			"keyDown")
(define xt:n-sticky-menus		"stickyMenus")
(define xt:n-top-shadow-color		"topShadowColor")
(define xt:n-bottom-shadow-color	"bottomShadowColor")
(define xt:n-bottom-shadow-tile		"bottomShadowTile")
(define xt:n-top-shadow-tile		"topShadowTile")
(define xt:n-allow-cascades		"allowCascades")
(define xt:n-pulldown-bar-id		"pulldownBarId")
(define xt:n-strip			"strip")
(define xt:n-title-precedence		"titlePrecedence")
(define xt:n-title-foreground		"titleForeground")
(define xt:n-title-background		"titleBackground")
(define xt:n-title-region		"titleRegion")
(define xt:n-title-position		"titlePosition")
(define xt:n-title-r-padding		"titleRPadding")
(define xt:n-title-l-padding		"titleLPadding")
(define xt:n-title-border-width		"titleBorderWidth")
(define xt:n-title-translations		"titleTranslations")
(define xt:n-title-h-space		"titleHSpace")
(define xt:n-title-v-space		"titleVSpace")
(define xt:n-title-select		"titleSelect")
(define xt:n-title-release		"titleRelease")
(define xt:n-title-next-top		"titleNextTop")
(define xt:n-titlebar-tile		"titlebarTile")
(define xt:n-enter			"enter")
(define xt:n-leave			"leave")
(define xt:n-region			"region")
(define xt:n-position			"position")
(define xt:n-l-padding			"lPadding")
(define xt:n-r-padding			"rPadding")
(define xt:n-precedence			"precedence")
(define xt:n-title-to-menu-pad		"titleToMenuPad")
(define xt:n-work-space-to-sibling-pad	"workSpaceToSiblingPad")
(define xt:n-widget-type		"widgetType")
(define xt:n-top-level			"topLevel")
(define xt:n-display-title		"displayTitle")
(define xt:n-causes-resize		"causesResize")
